@@ -1,8 +1,8 @@
 # python imports
-import re
 import math
 
 # django imports
+from django.conf import settings
 from django.core.cache import cache
 from django.db import connection
 
@@ -13,25 +13,18 @@ from lfs.catalog.settings import STANDARD_PRODUCT
 from lfs.catalog.settings import PRODUCT_WITH_VARIANTS
 from lfs.catalog.settings import PROPERTY_VALUE_TYPE_FILTER
 
-# TODO implement this methods.
-# Category
-def get_current_category_slug(request):
-    """Returns the current category.
-    """
-    pass
+# Load logger
+import logging
+logger = logging.getLogger("default")
 
-def get_current_category_id(request):
-    """Returns the current category.
-    """
-    pass
 
 # TODO: Add unit test
 def get_current_top_category(request, obj):
-    """Returns the current top category of a product.
     """
-
+    Returns the current top category of a product.
+    """
     if obj.__class__.__name__.lower() == "product":
-        category = get_current_product_category(request, obj)
+        category = obj.get_current_category(request)
     else:
         category = obj
 
@@ -43,26 +36,6 @@ def get_current_top_category(request, obj):
 
     return category
 
-def get_current_category(request):
-    """Returns the current category.
-    """
-    pass
-
-# Product
-def get_current_product_slug(request):
-    """Returns the current product id
-    """
-    pass
-
-def get_current_product_id(request):
-    """Returns the current product id
-    """
-    pass
-
-def get_current_product(request):
-    """Returns the current product
-    """
-    pass
 
 def get_current_product_category(request, product):
     """Returns product category based on actual categories of the given product
@@ -71,47 +44,16 @@ def get_current_product_category(request, product):
     This is needed if the category has more than one category to display
     breadcrumbs, selected menu points, etc. appropriately.
     """
-    try:
-        product_categories = product.get_categories()
-        if len(product_categories) == 1:
-            category = product_categories[0]
-        else:
-            last_category = request.session.get("last_category")
+    logger.info("Decprecated: lfs.catalog.utils: the function 'get_current_product_category' is deprecated. Please use 'get_current_category'of the Product class.")
+    return product.get_current_category(request)
 
-            if last_category is None:
-                return product_categories[0]
-
-            category = None
-            if last_category in product_categories:
-                category = last_category
-            else:
-                children = last_category.get_all_children()
-                for product_category in product_categories:
-                    if product_category in children:
-                        category = product_category
-                        break
-            if category is None:
-                category = product_categories[0]
-    except IndexError:
-        return None
-    else:
-        request.session["last_category"] = category
-        return category
 
 def get_property_groups(category):
     """Returns all property groups for given category
     """
-    cache_key = "category-property-groups-%s" % category.id
-    pgs = cache.get(cache_key)
-    if pgs is not None:
-        return pgs
+    logger.info("Decprecated: lfs.catalog.utils: the function 'get_property_groups' is deprecated. Please use 'get_property_groups'of the Category class.")
+    return category.get_property_groups()
 
-    products = category.get_products()
-    pgs = lfs.catalog.models.PropertyGroup.objects.filter(
-        products__in=products).distinct()
-
-    cache.set(cache_key, pgs)
-    return pgs
 
 def get_price_filters(category, product_filter, price_filter):
     """Creates price filter links based on the min and max price of the
@@ -125,8 +67,11 @@ def get_price_filters(category, product_filter, price_filter):
     # And their variants
     all_products = []
     for product in products:
-        all_products.append(product)
         all_products.extend(product.variants.filter(active=True))
+        if product.is_product_with_variants():
+            all_products.extend(product.variants.filter(active=True))
+        else:
+            all_products.append(product)
 
     product_ids = [p.id for p in all_products]
 
@@ -136,12 +81,11 @@ def get_price_filters(category, product_filter, price_filter):
         max = price_filter["max"]
         products = lfs.catalog.models.Product.objects.filter(
             effective_price__range=(min, max), pk__in=product_ids)
-        quantity = len(products)
 
         return {
-            "show_reset" : True,
-            "show_quantity" : False,
-            "items" : [{"min" : float(min), "max" : float(max)}],
+            "show_reset": True,
+            "show_quantity": False,
+            "items": [{"min": float(min), "max": float(max)}],
             }
 
     product_ids_str = ", ".join([str(p.id) for p in all_products])
@@ -180,13 +124,13 @@ def get_price_filters(category, product_filter, price_filter):
     for n, i in enumerate(range(0, int(pmax), step)):
         if i > pmax:
             break
-        min = i+1
-        max = i+step
+        min = i + 1
+        max = i + step
         products = lfs.catalog.models.Product.objects.filter(effective_price__range=(min, max), pk__in=product_ids)
         result.append({
-            "min" : min,
-            "max" : max,
-            "quantity" : len(products),
+            "min": min,
+            "max": max,
+            "quantity": len(products),
         })
 
     # return result
@@ -195,18 +139,19 @@ def get_price_filters(category, product_filter, price_filter):
     for n, f in enumerate(result):
         if f["quantity"] == 0:
             try:
-                result[n+1]["min"] = f["min"]
+                result[n + 1]["min"] = f["min"]
             except IndexError:
                 pass
             continue
         new_result.append(f)
 
     return {
-        "show_reset" : False,
-        "show_quantity" : True,
-        "items" : new_result,
+        "show_reset": False,
+        "show_quantity": True,
+        "items": new_result,
     }
 
+
 def get_product_filters(category, product_filter, price_filter, sorting):
     """Returns the next product filters based on products which are in the given
     category and within the result set of the current filters.
@@ -224,7 +169,7 @@ def get_product_filters(category, product_filter, price_filter, sorting):
     else:
         ck_product_filter = ""
 
-    cache_key = "productfilters-%s-%s-%s-%s" % (
+    cache_key = "%s-productfilters-%s-%s-%s-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX,
         category.slug, ck_product_filter, ck_price_filter, sorting)
 
     result = cache.get(cache_key)
@@ -252,14 +197,13 @@ def get_product_filters(category, product_filter, price_filter, sorting):
     # Create dict out of already set filters
     set_filters = dict(product_filter)
 
-
     cursor = connection.cursor()
     cursor.execute("""SELECT DISTINCT property_id
                       FROM catalog_productpropertyvalue""")
 
     property_ids = ", ".join([str(p[0]) for p in cursor.fetchall()])
 
-    # if there either no products or no property ids there can also be no
+    # if there is either no products or no property ids there can also be no
     # product filters.
     if not product_ids or not property_ids:
         return []
@@ -275,7 +219,6 @@ def get_product_filters(category, product_filter, price_filter, sorting):
                       AND property_id IN (%s)
                       GROUP BY property_id""" % (PROPERTY_VALUE_TYPE_FILTER, product_ids, property_ids))
 
-
     for row in cursor.fetchall():
 
         property = properties_mapping[row[0]]
@@ -291,31 +234,31 @@ def get_product_filters(category, product_filter, price_filter, sorting):
         if str(row[0]) in set_filters.keys():
             values = set_filters[str(row[0])]
             result.append({
-                "id" : row[0],
-                "position" : property.position,
-                "object" : property,
-                "name" : property.name,
-                "title" : property.title,
-                "unit" : property.unit,
-                "items" : [{"min" : float(values[0]), "max" : float(values[1])}],
-                "show_reset" : True,
-                "show_quantity" : False,
+                "id": row[0],
+                "position": property.position,
+                "object": property,
+                "name": property.name,
+                "title": property.title,
+                "unit": property.unit,
+                "items": [{"min": float(values[0]), "max": float(values[1])}],
+                "show_reset": True,
+                "show_quantity": False,
             })
             continue
 
         # Otherwise we display all steps.
-        items = calculate_steps(product_ids, property, row[1], row[2])
+        items = _calculate_steps(product_ids, property, row[1], row[2])
 
         result.append({
-            "id" : row[0],
-            "position" : property.position,
-            "object" : property,
-            "name" : property.name,
-            "title" : property.title,
-            "unit" : property.unit,
-            "show_reset" : False,
-            "show_quantity" : True,
-            "items" : items,
+            "id": row[0],
+            "position": property.position,
+            "object": property,
+            "name": property.name,
+            "title": property.title,
+            "unit": property.unit,
+            "show_reset": False,
+            "show_quantity": True,
+            "items": items,
         })
 
     ########## Select Fields ###################################################
@@ -339,14 +282,14 @@ def get_product_filters(category, product_filter, price_filter, sorting):
         # But we want to count color:red just one time. As the product with
         # variants is displayed at not the variants.
 
-        if already_count.has_key("%s%s%s" % (row[2], row[0], row[1])):
+        if "%s%s%s" % (row[2], row[0], row[1]) in already_count:
             continue
         already_count["%s%s%s" % (row[2], row[0], row[1])] = 1
 
-        if not amount.has_key(row[0]):
+        if row[0] not in amount:
             amount[row[0]] = {}
 
-        if not amount[row[0]].has_key(row[1]):
+        if row[1] not in amount[row[0]]:
             amount[row[0]][row[1]] = 0
 
         amount[row[0]][row[1]] += 1
@@ -371,23 +314,22 @@ def get_product_filters(category, product_filter, price_filter, sorting):
         if property.filterable == False:
             continue
 
-        if properties.has_key(row[0]) == False:
+        if row[0] in properties == False:
             properties[row[0]] = []
 
         # If the property is a select field we want to display the name of the
         # option instead of the id.
-        position = 1
-        if properties_mapping[row[0]].is_select_field:
+        if property.is_select_field:
             try:
                 name = options_mapping[row[1]].name
-                position = options_mapping[row[1]].position
             except KeyError:
                 name = row[1]
+        elif property.is_number_field:
+            value = float(row[1])
         else:
             name = row[1]
 
         # Transform to float for later sorting, see below
-        property = properties_mapping[row[0]]
         if property.is_number_field:
             value = float(row[1])
         else:
@@ -397,22 +339,22 @@ def get_product_filters(category, product_filter, price_filter, sorting):
         if str(row[0]) in set_filters.keys():
             if str(row[1]) in set_filters.values():
                 properties[row[0]] = [{
-                    "id"       : row[0],
-                    "value"    : value,
-                    "name"     : name,
-                    "position": position,
-                    "quantity" : amount[row[0]][row[1]],
-                    "show_quantity" : False,
+                    "id": row[0],
+                    "value": value,
+                    "name": name,
+                    "quantity": amount[row[0]][row[1]],
+                    "show_quantity": False,
                 }]
             continue
         else:
+            if not properties.has_key(row[0]):
+                properties[row[0]] = []
             properties[row[0]].append({
-                "id"       : row[0],
-                "value"    : value,
-                "name"     : name,
-                "position": position,
-                "quantity" : amount[row[0]][row[1]],
-                "show_quantity" : True,
+                "id": row[0],
+                "value": value,
+                "name": name,
+                "quantity": amount[row[0]][row[1]],
+                "show_quantity": True,
             })
 
     # Transform the group properties into a list of dicts
@@ -425,16 +367,16 @@ def get_product_filters(category, product_filter, price_filter, sorting):
         # Sort the values. NOTE: This has to be done here (and not via SQL) as
         # the value field of the property is a char field and can't ordered
         # properly for numbers.
-        values.sort(lambda a, b: cmp(a["position"], b["position"]))
+        values.sort(lambda a, b: cmp(a["value"], b["value"]))
 
         result.append({
-            "id"    : property_id,
-            "position" : property.position,
-            "unit" : property.unit,
-            "show_reset" : str(property_id) in set_filter_keys,
-            "name"  : property.name,
-            "title" : property.title,
-            "items" : values
+            "id": property_id,
+            "position": property.position,
+            "unit": property.unit,
+            "show_reset": str(property_id) in set_filter_keys,
+            "name": property.name,
+            "title": property.title,
+            "items": values,
         })
 
     result.sort(lambda a, b: cmp(a["position"], b["position"]))
@@ -442,6 +384,8 @@ def get_product_filters(category, product_filter, price_filter, sorting):
 
     return result
 
+
+# TODO: Implement this as a method of Category
 def get_filtered_products_for_category(category, filters, price_filter, sorting):
     """Returns products for given categories and current filters sorted by
     current sorting.
@@ -494,7 +438,7 @@ def get_filtered_products_for_category(category, filters, price_filter, sorting)
                 FROM catalog_productpropertyvalue
                 WHERE product_id IN (%s) and %s and type=%s
                 GROUP BY product_id
-                HAVING count(*)=%s""" % (all_variant_ids, PROPERTY_VALUE_TYPE_FILTER, fstr, len(filters)))
+                HAVING count(*)=%s""" % (all_variant_ids, fstr, PROPERTY_VALUE_TYPE_FILTER, len(filters)))
 
             # Get the parent ids of the variants as the "product with variants"
             # should be displayed and not the variants.
@@ -560,14 +504,16 @@ def get_filtered_products_for_category(category, filters, price_filter, sorting)
 
     return products
 
+
 def get_option_mapping():
-    """Returns a dictionary with property id to property name.
+    """Returns a dictionary with option id to property name.
     """
     options = {}
     for option in lfs.catalog.models.PropertyOption.objects.all():
         options[str(option.id)] = option
     return options
 
+
 def get_property_mapping():
     """Returns a dictionary with property id to property name.
     """
@@ -577,8 +523,21 @@ def get_property_mapping():
 
     return properties
 
-def calculate_steps(product_ids, property, min, max):
-    """
+
+def _calculate_steps(product_ids, property, min, max):
+    """Calculates filter steps.
+
+    **Parameters**
+
+    product_ids
+        The product_ids for which the steps are calculated. List of ids.
+
+    property
+        The property for which the steps are calculated. Instance of Property.
+
+    min / max
+        The min and max value of all steps. Must be a Float.
+
     """
     try:
         min = float(min)
@@ -590,16 +549,16 @@ def calculate_steps(product_ids, property, min, max):
 
     filter_steps = lfs.catalog.models.FilterStep.objects.filter(property=property.id)
     if property.is_steps_step_type:
-        for i, step in enumerate(filter_steps[:len(filter_steps)-1]):
+        for i, step in enumerate(filter_steps[:len(filter_steps) - 1]):
             min = step.start
             if i != 0:
                 min += 1.0
-            max = filter_steps[i+1].start
+            max = filter_steps[i + 1].start
 
             result.append({
-                "min" : min,
-                "max" : max,
-                "quantity" : calculate_quantity(product_ids, property.id, min, max)
+                "min": min,
+                "max": max,
+                "quantity": _calculate_quantity(product_ids, property.id, min, max)
             })
     else:
         if property.is_automatic_step_type:
@@ -633,13 +592,13 @@ def calculate_steps(product_ids, property, min, max):
         for n, i in enumerate(range(0, int(max), step)):
             if i > max:
                 break
-            min = i+1
-            max = i+step
+            min = i + 1
+            max = i + step
 
             result.append({
-                "min" : min,
-                "max" : max,
-                "quantity" : calculate_quantity(product_ids, property.id, min, max)
+                "min": min,
+                "max": max,
+                "quantity": _calculate_quantity(product_ids, property.id, min, max),
             })
 
     if property.display_no_results:
@@ -650,7 +609,7 @@ def calculate_steps(product_ids, property, min, max):
         for n, f in enumerate(result):
             if f["quantity"] == 0:
                 try:
-                    result[n+1]["min"] = f["min"]
+                    result[n + 1]["min"] = f["min"]
                 except IndexError:
                     pass
                 continue
@@ -658,7 +617,8 @@ def calculate_steps(product_ids, property, min, max):
 
         return new_result
 
-def calculate_quantity(product_ids, property_id, min, max):
+
+def _calculate_quantity(product_ids, property_id, min, max):
     """Calculate the amount of products for given parameters.
     """
     # Count entries for current filter
@@ -681,7 +641,7 @@ def calculate_quantity(product_ids, property_id, min, max):
         # But we want to count color:red just one time. As the product with
         # variants is displayed at not the variants.
 
-        if already_count.has_key("%s%s%s" % (row[2], row[0], row[1])):
+        if "%s%s%s" % (row[2], row[0], row[1]) in already_count:
             continue
         already_count["%s%s%s" % (row[2], row[0], row[1])] = 1
 
@@ -689,13 +649,17 @@ def calculate_quantity(product_ids, property_id, min, max):
 
     return amount
 
+
 def calculate_packages(product, quantity):
+    """Returns amount of packages passed on passes product and quantity.
+    DEPRECATED.
     """
-    """
+    logger.info("Decprecated: lfs.catalog.utils: the function 'calculate_packages' is deprecated.")
     return math.ceil(quantity / product.packing_unit)
 
+
 def calculate_real_amount(product, quantity):
+    """Returns the amount of pieces in package units. DEPRECATED.
     """
-    """
-    packages = calculate_packages(product, quantity)
-    return packages * product.packing_unit
+    logger.info("Decprecated: lfs.catalog.utils: the function 'calculate_real_amount' is deprecated. Please use 'get_amount_by_packages'of the Product class.")
+    return product.get_amount_by_packages(quantity)
