@@ -34,6 +34,7 @@ from lfs.core.utils import LazyEncoder
 from lfs.core.templatetags import lfs_tags
 from lfs.utils import misc as lfs_utils
 
+
 def file(request, language=None, id=None):
     """Delivers files to the browser.
     """
@@ -43,6 +44,7 @@ def file(request, language=None, id=None):
 
     return response
 
+
 def select_variant(request):
     """This is called via an ajax call if the combination of properties are
     changed.
@@ -51,48 +53,51 @@ def select_variant(request):
     msg = _(u"The product has been changed according to your selection.")
 
     result = simplejson.dumps({
-        "product" : product_inline(request, variant_id),
-        "message" : msg,
-    }, cls = LazyEncoder)
+        "product": product_inline(request, variant_id),
+        "message": msg,
+    }, cls=LazyEncoder)
 
     return HttpResponse(result)
 
+
 def calculate_packing(request, id, quantity=None, as_string=False, template_name="lfs/catalog/packing_result.html"):
     """Calculates the actual amount of pieces to buy on base on packing
     information.
     """
-    product = Product.objects.get(pk = id)
+    product = Product.objects.get(pk=id)
 
     if quantity is None:
         quantity = float(request.POST.get("quantity"))
 
-    packing_amount, packing_unit= product.get_packing_info()
+    packing_amount, packing_unit = product.get_packing_info()
 
     packs = math.ceil(quantity / packing_amount)
     real_quantity = packs * packing_amount
-    price = real_quantity * product.get_price()
+    price = real_quantity * product.get_price(request)
 
     html = render_to_string(template_name, RequestContext(request, {
-        "price" : price,
-        "product" : product,
-        "packs" : int(packs),
-        "real_quantity" : real_quantity,
-        "unit" : packing_unit,
+        "price": price,
+        "product": product,
+        "packs": int(packs),
+        "real_quantity": real_quantity,
+        "unit": packing_unit,
     }))
 
     if as_string:
         return html
 
     result = simplejson.dumps({
-        "html" : html,
-    }, cls = LazyEncoder)
+        "html": html,
+    }, cls=LazyEncoder)
 
     return HttpResponse(result)
 
+
 def calculate_price(request, id):
-    """Calculates the price of the product on base of choosen properties.
+    """Calculates the price of the product on base of choosen properties after
+    a customer has selected a property on product view.
     """
-    product = Product.objects.get(pk = id)
+    product = Product.objects.get(pk=id)
 
     property_price = 0
     for key, option_id in request.POST.items():
@@ -105,24 +110,23 @@ def calculate_price(request, id):
                 if po.property.add_price:
                     property_price += po.price
 
-    for_sale_price = product.get_for_sale_price(with_properties=False)
-    for_sale_price += property_price
-
-    for_sale_standard_price = product.get_standard_price(with_properties=False)
-    for_sale_standard_price += property_price
-
-    price = product.get_price(with_properties=False)
-    price += property_price
+    if product.for_sale:
+        for_sale_standard_price = product.get_standard_price(with_properties=False)
+        for_sale_standard_price += property_price
+    else:
+        for_sale_standard_price = 0
+        price = product.get_price(request, with_properties=False)
+        price += property_price
 
     result = simplejson.dumps({
-        "price" : lfs_tags.currency(price),
-        "for-sale-price" : lfs_tags.currency(for_sale_price),
-        "for-sale-standard-price" : lfs_tags.currency(for_sale_standard_price),
-        "message" : _("Price has been changed according to your selection."),
-    }, cls = LazyEncoder)
+        "price": lfs_tags.currency(price),
+        "for-sale-standard-price": lfs_tags.currency(for_sale_standard_price),
+        "message": _("Price has been changed according to your selection."),
+    }, cls=LazyEncoder)
 
     return HttpResponse(result)
 
+
 def select_variant_from_properties(request):
     """This is called via an ajax call if the combination of properties are
     changed.
@@ -130,8 +134,8 @@ def select_variant_from_properties(request):
     product_id = request.POST.get("product_id")
 
     try:
-        product = Product.objects.get(pk = product_id)
-    except:# Product.DoesNotExist:
+        product = Product.objects.get(pk=product_id)
+    except:  # Product.DoesNotExist:
         return HttpResponse("")
 
     options = lfs_utils.parse_properties(request)
@@ -144,12 +148,13 @@ def select_variant_from_properties(request):
         msg = _(u"The product has been changed according to your selection.")
 
     result = simplejson.dumps({
-        "product" : product_inline(request, variant.id),
-        "message" : msg,
-    }, cls = LazyEncoder)
+        "product": product_inline(request, variant.id),
+        "message": msg,
+    }, cls=LazyEncoder)
 
     return HttpResponse(result)
 
+
 def set_filter(request, category_slug, property_id, value=None, min=None, max=None):
     """Saves the given filter to session. Redirects to the category with given
     slug.
@@ -163,9 +168,10 @@ def set_filter(request, category_slug, property_id, value=None, min=None, max=No
 
     request.session["product-filter"] = product_filter
 
-    url = reverse("lfs_category", kwargs={"slug" : category_slug })
+    url = reverse("lfs_category", kwargs={"slug": category_slug})
     return HttpResponseRedirect(url)
 
+
 def set_price_filter(request, category_slug):
     """Saves the given price filter to session. Redirects to the category with
     given slug.
@@ -183,49 +189,53 @@ def set_price_filter(request, category_slug):
     except TypeError:
         max = "0"
 
-    request.session["price-filter"] = {"min" : min, "max": max}
+    request.session["price-filter"] = {"min": min, "max": max}
 
-    url = reverse("lfs_category", kwargs={"slug" : category_slug })
+    url = reverse("lfs_category", kwargs={"slug": category_slug})
     return HttpResponseRedirect(url)
 
+
 def reset_price_filter(request, category_slug):
     """Resets the price filter. Redirects to the category with given slug.
     """
-    if request.session.has_key("price-filter"):
+    if "price-filter" in request.session:
         del request.session["price-filter"]
 
-    url = reverse("lfs_category", kwargs={"slug" : category_slug })
+    url = reverse("lfs_category", kwargs={"slug": category_slug})
     return HttpResponseRedirect(url)
 
+
 def reset_filter(request, category_slug, property_id):
     """Resets product filter with given property id. Redirects to the category
     with given slug.
     """
-    if request.session.has_key("product-filter"):
-        if request.session["product-filter"].has_key(property_id):
+    if "product-filter" in request.session:
+        if property_id in request.session["product-filter"]:
             del request.session["product-filter"][property_id]
             request.session["product-filter"] = request.session["product-filter"]
 
-    url = reverse("lfs_category", kwargs={"slug" : category_slug })
+    url = reverse("lfs_category", kwargs={"slug": category_slug})
     return HttpResponseRedirect(url)
 
+
 def reset_all_filter(request, category_slug):
     """Resets all product filter. Redirects to the category with given slug.
     """
-    if request.session.has_key("product-filter"):
+    if "product-filter" in request.session:
         del request.session["product-filter"]
 
-    if request.session.has_key("price-filter"):
+    if "price-filter" in request.session:
         del request.session["price-filter"]
 
-    url = reverse("lfs_category", kwargs={"slug" : category_slug })
+    url = reverse("lfs_category", kwargs={"slug": category_slug})
     return HttpResponseRedirect(url)
 
+
 def set_sorting(request):
     """Saves the given sortings (by request body) to session.
     """
     sorting = request.POST.get("sorting", "")
-    if sorting == "" and request.session.has_key("sorting"):
+    if sorting == "" and "sorting" in request.session:
         del request.session["sorting"]
     else:
         request.session["sorting"] = sorting
@@ -233,6 +243,7 @@ def set_sorting(request):
     # lfs_sorting_changed.send(category_id)
     return HttpResponseRedirect(request.META.get("HTTP_REFERER"))
 
+
 def category_view(request, slug, template_name="lfs/catalog/category_base.html"):
     """
     """
@@ -250,17 +261,18 @@ def category_view(request, slug, template_name="lfs/catalog/category_base.html")
     # omit if they don't need it.
 
     return render_to_response(template_name, RequestContext(request, {
-        "category" : category,
-        "category_inline" : inline,
-        "top_category" : lfs.catalog.utils.get_current_top_category(request, category),
+        "category": category,
+        "category_inline": inline,
+        "top_category": lfs.catalog.utils.get_current_top_category(request, category),
     }))
 
+
 def category_categories(request, slug, start=0, template_name="lfs/catalog/categories/category/default.html"):
     """Displays the child categories of the category with passed slug.
 
     This view is called if the user chooses a template that is situated in settings.CATEGORY_PATH ".
     """
-    cache_key = "category-categories-%s" % slug
+    cache_key = "%s-category-categories-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX, slug)
 
     result = cache.get(cache_key)
     if result is not None:
@@ -275,7 +287,7 @@ def category_categories(request, slug, start=0, template_name="lfs/catalog/categ
     row = []
     for i, children in enumerate(category.get_children()):
         row.append(children)
-        if (i+1) % amount_of_cols == 0:
+        if (i + 1) % amount_of_cols == 0:
             categories.append(row)
             row = []
 
@@ -287,13 +299,14 @@ def category_categories(request, slug, start=0, template_name="lfs/catalog/categ
         template_name = render_template
 
     result = render_to_string(template_name, RequestContext(request, {
-        "category" : category,
-        "categories" : categories,
+        "category": category,
+        "categories": categories,
     }))
 
     cache.set(cache_key, result)
     return result
 
+
 def category_products(request, slug, start=0, template_name="lfs/catalog/categories/product/default.html"):
     """Displays the products of the category with passed slug.
 
@@ -303,9 +316,9 @@ def category_products(request, slug, start=0, template_name="lfs/catalog/categor
     # TODO: Is this what a customer would expect?
     last_category = request.session.get("last_category")
     if (last_category is None) or (last_category.slug != slug):
-        if request.session.has_key("product-filter"):
+        if "product-filter" in request.session:
             del request.session["product-filter"]
-        if request.session.has_key("price-filter"):
+        if "price-filter" in request.session:
             del request.session["price-filter"]
 
     try:
@@ -317,8 +330,8 @@ def category_products(request, slug, start=0, template_name="lfs/catalog/categor
     product_filter = request.session.get("product-filter", {})
     product_filter = product_filter.items()
 
-    cache_key = "category-products-%s" % slug
-    sub_cache_key = "start-%s-sorting-%s" % (start, sorting)
+    cache_key = "%s-category-products-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX, slug)
+    sub_cache_key = "%s-start-%s-sorting-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX, start, sorting)
 
     filter_key = ["%s-%s" % (i[0], i[1]) for i in product_filter]
     if filter_key:
@@ -356,19 +369,25 @@ def category_products(request, slug, start=0, template_name="lfs/catalog/categor
     # Calculate products
     row = []
     products = []
-    for i, product in enumerate(all_products[start:start+amount]):
+    for i, product in enumerate(all_products[start:start + amount]):
         # Switch to default variant if product is a "product with variants".
         # if product.is_product_with_variants() and product.has_variants():
         #    product = product.get_default_variant()
+        image = None
+        product_image = product.get_image()
+        if product_image:
+            image = product_image.image
         row.append({
-            "obj" : product,
-            "slug" : product.slug,
-            "name" : product.get_name(),
-            "image" : product.get_image(),
-            "price" : product.get_price(),
-            "standard_price" : product.get_standard_price(),
+            "obj": product,
+            "slug": product.slug,
+            "name": product.get_name(),
+            "image": image,
+            "price": product.get_price(request),
+            "standard_price": product.get_standard_price(request),
+            "price_unit": product.price_unit,
+            "price_includes_tax": product.price_includes_tax,
         })
-        if (i+1) % amount_of_cols == 0:
+        if (i + 1) % amount_of_cols == 0:
             products.append(row)
             row = []
 
@@ -379,12 +398,12 @@ def category_products(request, slug, start=0, template_name="lfs/catalog/categor
 
     # Calculate urls
     pages = []
-    for i in range(0, amount_of_products/amount+1):
-        page_start = i*amount
+    for i in range(0, amount_of_products / amount + 1):
+        page_start = i * amount
         pages.append({
-            "name" : i+1,
-            "start" : page_start,
-            "selected" : start == page_start,
+            "name": i + 1,
+            "start": page_start,
+            "selected": start == page_start,
         })
 
     if (start + amount) < amount_of_products:
@@ -402,20 +421,21 @@ def category_products(request, slug, start=0, template_name="lfs/catalog/categor
         template_name = render_template
 
     result = render_to_string(template_name, RequestContext(request, {
-        "category" : category,
-        "products" : products,
-        "next_url" : next_url,
-        "previous_url" : previous_url,
-        "amount_of_products" : amount_of_products,
-        "pages" : pages,
-        "show_pages" : amount_of_products > amount,
-        "all_products" : all_products
+        "category": category,
+        "products": products,
+        "next_url": next_url,
+        "previous_url": previous_url,
+        "amount_of_products": amount_of_products,
+        "pages": pages,
+        "show_pages": amount_of_products > amount,
+        "all_products": all_products,
     }))
 
     temp[sub_cache_key] = result
     cache.set(cache_key, temp)
     return result
 
+
 def product_view(request, slug, template_name="lfs/catalog/product_base.html"):
     """Main view to display a product.
     """
@@ -430,26 +450,24 @@ def product_view(request, slug, template_name="lfs/catalog/product_base.html"):
         recent.remove(slug)
     recent.insert(0, slug)
     if len(recent) > settings.LFS_RECENT_PRODUCTS_LIMIT:
-        recent = recent[:settings.LFS_RECENT_PRODUCTS_LIMIT+1]
+        recent = recent[:settings.LFS_RECENT_PRODUCTS_LIMIT + 1]
     request.session["RECENT_PRODUCTS"] = recent
 
-    # TODO: Factor top_category out to a inclusion tag, so that people can
-    # omit if they don't need it.
-    result =  render_to_response(template_name, RequestContext(request, {
-        "product_inline" : product_inline(request, product.id),
-        "product" : product,
-        "top_category" : lfs.catalog.utils.get_current_top_category(request, product),
+    result = render_to_response(template_name, RequestContext(request, {
+        "product_inline": product_inline(request, product.id),
+        "product": product,
     }))
 
     return result
 
+
 def product_inline(request, id, template_name="lfs/catalog/products/product_inline.html"):
     """Part of the prduct view, which displays the actual data of the product.
 
     This is factored out to be able to better cached and in might in future used
     used to be updated via ajax requests.
     """
-    cache_key = "product-inline-%s-%s" % (request.user.is_superuser, id)
+    cache_key = "%s-product-inline-%s-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX, request.user.is_superuser, id)
     result = cache.get(cache_key)
     if result is not None:
         return result
@@ -474,7 +492,7 @@ def product_inline(request, id, template_name="lfs/catalog/products/product_inli
         if product.variants_display_type == SELECT:
             # Get all properties (sorted). We need to traverse through all
             # property/options to select the options of the current variant.
-            for property in product.get_properties():
+            for property in product.get_property_select_fields():
                 options = []
                 for property_option in property.options.all():
                     if variant.has_option(property, property_option):
@@ -482,22 +500,22 @@ def product_inline(request, id, template_name="lfs/catalog/products/product_inli
                     else:
                         selected = False
                     options.append({
-                        "id"   : property_option.id,
-                        "name" : property_option.name,
-                        "selected" : selected
+                        "id": property_option.id,
+                        "name": property_option.name,
+                        "selected": selected,
                     })
                 properties.append({
-                    "id" : property.id,
-                    "name" : property.name,
-                    "title" : property.title,
-                    "unit" : property.unit,
-                    "options" : options,
+                    "id": property.id,
+                    "name": property.name,
+                    "title": property.title,
+                    "unit": property.unit,
+                    "options": options,
                 })
         else:
-            properties = product.get_properties()
+            properties = product.get_property_select_fields()
             variants = product.get_variants()
 
-    elif product.is_configurable_product:
+    elif product.is_configurable_product():
         for property in product.get_configurable_properties():
             options = []
 
@@ -507,57 +525,54 @@ def product_inline(request, id, template_name="lfs/catalog/products/product_inli
             except ProductPropertyValue.DoesNotExist:
                 ppv = None
                 ppv_value = ""
-                
+
             for property_option in property.options.all():
                 if ppv_value == str(property_option.id):
-                    selected = True                    
+                    selected = True
                 else:
                     selected = False
 
                 options.append({
-                    "id"   : property_option.id,
-                    "name" : property_option.name,
-                    "price" : property_option.price,
-                    "selected" : selected,
+                    "id": property_option.id,
+                    "name": property_option.name,
+                    "price": property_option.price,
+                    "selected": selected,
                 })
 
             properties.append({
-                "obj" : property,
-                "id" : property.id,
-                "name" : property.name,
-                "title" : property.title,
-                "unit" : property.unit,
-                "display_price" : property.display_price,
-                "options" : options,
-                "value" : ppv_value,
+                "obj": property,
+                "id": property.id,
+                "name": property.name,
+                "title": property.title,
+                "unit": property.unit,
+                "display_price": property.display_price,
+                "options": options,
+                "value": ppv_value,
             })
 
-    # Reviews
     if product.get_template_name() != None:
         template_name = product.get_template_name()
 
-    message = request.COOKIES.get("message")
-    if message:
-        message = urllib.unquote(message)
-
-    if product.active_packing_unit:
-        packing_result = calculate_packing(request, id, 1, True)
+    if variant.active_packing_unit:
+        packing_result = calculate_packing(request, variant.id, 1, True)
     else:
         packing_result = ""
 
     result = render_to_string(template_name, RequestContext(request, {
-        "product" : product,
-        "variant" : variant,
-        "variants" : variants,
-        "product_accessories" : variant.get_accessories(),
-        "properties" : properties,
-        "message" : message,
-        "packing_result" : packing_result,
+        "product": product,
+        "standard_price": variant.get_standard_price(request),
+        "price": variant.get_price(request),
+        "variant": variant,
+        "variants": variants,
+        "product_accessories": variant.get_accessories(),
+        "properties": properties,
+        "packing_result": packing_result,
     }))
 
     cache.set(cache_key, result)
     return result
 
+
 def product_form_dispatcher(request):
     """Dispatches to the added-to-cart view or to the selected variant.
 
@@ -571,7 +586,7 @@ def product_form_dispatcher(request):
         return add_to_cart(request)
     else:
         product_id = request.POST.get("product_id")
-        product = Product.objects.get(pk=product_id)
+        product = lfs_get_object_or_404(Product, pk=product_id)
 
         options = lfs_utils.parse_properties(request)
         variant = product.get_variant(options)
@@ -581,38 +596,7 @@ def product_form_dispatcher(request):
 
             return lfs.core.utils.set_message_cookie(
                 variant.get_absolute_url(),
-                msg = _(u"The choosen combination of properties is not deliverable.")
+                msg=_(u"The choosen combination of properties is not deliverable.")
             )
 
         return HttpResponseRedirect(variant.get_absolute_url())
-
-# NOT used at moment
-def get_category_nodes(request):
-    """Returns the category tree as JSON for extJS.
-    """
-    categories = []
-    for category in Category.objects.filter(parent = None):
-        temp = _get_children_nodes(category)
-        categories.append({
-            "id" : category.slug,
-            "text" : category.name,
-            "leaf" : len(temp) == 0,
-            "children" : temp
-        })
-
-    return HttpResponse(simplejson.dumps(categories))
-
-def _get_children_nodes(category):
-    """
-    """
-    children = []
-    for category in category.category_set.all():
-        temp = _get_children_nodes(category)
-        children.append({
-            "id" : category.slug,
-            "text" : category.name,
-            "leaf" : len(temp) == 0,
-            "children" : temp,
-        })
-
-    return children
