@@ -5,7 +5,7 @@ from datetime import datetime
 from django.contrib.sitemaps import Sitemap
 from lfs.catalog.models import Category
 from lfs.catalog.models import Product
-from lfs.catalog.settings import PRODUCT_WITH_VARIANTS
+
 
 class ProductSitemap(Sitemap):
     """Google's XML sitemap for products.
@@ -14,11 +14,12 @@ class ProductSitemap(Sitemap):
     priority = 0.5
 
     def items(self):
-        return Product.objects.filter(active=True).exclude(sub_type=PRODUCT_WITH_VARIANTS)
+        return Product.objects.filter(active=True)
 
     def lastmod(self, obj):
         return obj.creation_date
-        
+
+
 class CategorySitemap(Sitemap):
     """Google's XML sitemap for products.
     """
@@ -29,4 +30,4 @@ class CategorySitemap(Sitemap):
         return Category.objects.all()
 
     def lastmod(self, obj):
-        return datetime.now()
\ No newline at end of file
+        return datetime.now()
