@@ -1,6 +1,6 @@
 # django imports
-from django.core.urlresolvers import reverse
 from django.contrib.auth.decorators import permission_required
+from django.core.urlresolvers import reverse
 from django.forms import ModelForm
 from django.http import HttpResponse
 from django.http import HttpResponseRedirect
@@ -10,6 +10,7 @@ from django.template import RequestContext
 from django.template.loader import render_to_string
 from django.utils import simplejson
 from django.utils.translation import ugettext_lazy as _
+from django.views.decorators.http import require_POST
 
 # lfs imports
 import lfs.core.utils
@@ -21,15 +22,16 @@ from lfs.core.utils import LazyEncoder
 from lfs.manufacturer.models import Manufacturer
 from lfs.manufacturer.models import Manufacturer
 
+
 class ManufacturerDataForm(ModelForm):
     """Form to manage selection data.
     """
     class Meta:
         model = Manufacturer
 
-@permission_required("core.manage_shop", login_url="/login/")
+
 def manage_manufacturer(request, manufacturer_id, template_name="manage/manufacturer/manufacturer.html"):
-    """The main view to display factories.
+    """The main view to display manufacturers.
     """
     manufacturer = Manufacturer.objects.get(pk=manufacturer_id)
 
@@ -40,41 +42,42 @@ def manage_manufacturer(request, manufacturer_id, template_name="manage/manufact
         checked, klass = _get_category_state(manufacturer, category)
 
         categories.append({
-            "id" : category.id,
-            "name" : category.name,
-            "checked" : checked,
-            "klass" : klass,
+            "id": category.id,
+            "name": category.name,
+            "checked": checked,
+            "klass": klass,
         })
 
     data_form = ManufacturerDataForm(instance=manufacturer)
     return render_to_response(template_name, RequestContext(request, {
-        "categories" : categories,
-        "manufacturer_id" : manufacturer_id,
-        "selectable_manufacturers_inline" : selectable_manufacturers_inline(request, manufacturer_id),
-        "manufacturer_data_inline" : manufacturer_data_inline(request, manufacturer_id, data_form),
+        "categories": categories,
+        "manufacturer_id": manufacturer_id,
+        "selectable_manufacturers_inline": selectable_manufacturers_inline(request, manufacturer_id),
+        "manufacturer_data_inline": manufacturer_data_inline(request, manufacturer_id, data_form),
     }))
 
+
 # Parts
 def manufacturer_data_inline(request, manufacturer_id, form,
     template_name="manage/manufacturer/manufacturer_data_inline.html"):
     """Displays the data form of the current manufacturer.
     """
     return render_to_string(template_name, RequestContext(request, {
-        "manufacturer_id" : manufacturer_id,
-        "form" : form,
+        "manufacturer_id": manufacturer_id,
+        "form": form,
     }))
 
+
 def selectable_manufacturers_inline(request, manufacturer_id,
     template_name="manage/manufacturer/selectable_manufacturers_inline.html"):
-    """Displays all selectable factories.
+    """Displays all selectable manufacturers.
     """
     return render_to_string(template_name, RequestContext(request, {
-        "factories" : Manufacturer.objects.all(),
-        "manufacturer_id" : int(manufacturer_id),
+        "manufacturers": Manufacturer.objects.all(),
+        "manufacturer_id": int(manufacturer_id),
     }))
 
 
-@permission_required("core.manage_shop", login_url="/login/")
 def manufacturer_inline(request, manufacturer_id, category_id,
     template_name="manage/manufacturer/manufacturer_inline.html"):
     """Returns categories and products for given manufacturer id and category id.
@@ -91,10 +94,10 @@ def manufacturer_inline(request, manufacturer_id, category_id,
             type = "V"
 
         products.append({
-            "id" : product.id,
-            "name" : product.get_name(),
-            "checked" : product in selected_products,
-            "type" : type,
+            "id": product.id,
+            "name": product.get_name(),
+            "checked": product in selected_products,
+            "type": type,
         })
 
     categories = []
@@ -103,25 +106,24 @@ def manufacturer_inline(request, manufacturer_id, category_id,
         checked, klass = _get_category_state(manufacturer, category)
 
         categories.append({
-            "id" : category.id,
-            "name" : category.name,
-            "checked" : checked,
-            "klass" : klass,
+            "id": category.id,
+            "name": category.name,
+            "checked": checked,
+            "klass": klass,
         })
 
     result = render_to_string(template_name, RequestContext(request, {
-        "categories" : categories,
-        "products" : products,
-        "manufacturer_id" : manufacturer_id,
+        "categories": categories,
+        "products": products,
+        "manufacturer_id": manufacturer_id,
     }))
 
     html = (("#sub-categories-%s" % category_id, result),)
 
     return HttpResponse(
-        simplejson.dumps({ "html" : html }))
+        simplejson.dumps({"html": html}))
 
 
-@permission_required("core.manage_shop", login_url="/login/")
 def add_manufacturer(request, template_name="manage/manufacturer/add_manufacturer.html"):
     """Form and logic to add a manufacturer.
     """
@@ -130,19 +132,19 @@ def add_manufacturer(request, template_name="manage/manufacturer/add_manufacture
         if form.is_valid():
             new_manufacturer = form.save()
             return HttpResponseRedirect(
-                reverse("lfs_manufacturer", kwargs={ "manufacturer_id" : new_manufacturer.id }))
+                reverse("lfs_manufacturer", kwargs={"manufacturer_id": new_manufacturer.id}))
 
     else:
         form = ManufacturerDataForm()
 
     return render_to_response(template_name, RequestContext(request, {
-        "form" : form,
-        "selectable_manufacturers_inline" : selectable_manufacturers_inline(request, 0),
+        "form": form,
+        "selectable_manufacturers_inline": selectable_manufacturers_inline(request, 0),
+        "next": request.REQUEST.get("next", request.META.get("HTTP_REFERER")),
     }))
 
 
 # Actions
-@permission_required("core.manage_shop", login_url="/login/")
 def manufacturer_dispatcher(request):
     """Dispatches to the first manufacturer or to the add form.
     """
@@ -152,9 +154,10 @@ def manufacturer_dispatcher(request):
         return HttpResponseRedirect(reverse("lfs_manufacturer_add_manufacturer"))
     else:
         return HttpResponseRedirect(
-            reverse("lfs_manufacturer", kwargs = {"manufacturer_id" : manufacturer.id }))
+            reverse("lfs_manufacturer", kwargs={"manufacturer_id": manufacturer.id}))
 
 
+@require_POST
 @permission_required("core.manage_shop", login_url="/login/")
 def delete_manufacturer(request, manufacturer_id):
     """Deletes Manufacturer with passed manufacturer id.
@@ -169,7 +172,6 @@ def delete_manufacturer(request, manufacturer_id):
     return HttpResponseRedirect(reverse("lfs_manufacturer_dispatcher"))
 
 
-@permission_required("core.manage_shop", login_url="/login/")
 def edit_category(request, manufacturer_id, category_id):
     """Adds/Removes products of given category to given manufacturer.
     """
@@ -188,7 +190,6 @@ def edit_category(request, manufacturer_id, category_id):
     return HttpResponse("")
 
 
-@permission_required("core.manage_shop", login_url="/login/")
 def edit_product(request, manufacturer_id, product_id):
     """Adds/Removes given product to given manufacturer.
     """
@@ -205,7 +206,6 @@ def edit_product(request, manufacturer_id, product_id):
     return HttpResponse("")
 
 
-@permission_required("core.manage_shop", login_url="/login/")
 def category_state(request, manufacturer_id, category_id):
     """Sets the state (klass and checking) for given category for given
     manufacturer.
@@ -224,13 +224,12 @@ def category_state(request, manufacturer_id, category_id):
 
     return HttpResponse(
         simplejson.dumps({
-            "html" : html,
-            "checkbox" : checkbox
+            "html": html,
+            "checkbox": checkbox
         })
     )
 
 
-@permission_required("core.manage_shop", login_url="/login/")
 def update_data(request, manufacturer_id):
     """Updates data of manufacturer with given manufacturer id.
     """
@@ -244,13 +243,13 @@ def update_data(request, manufacturer_id):
 
     html = (
         ("#data", manufacturer_data_inline(request, manufacturer_id, form)),
-        ("#selectable-factories-inline", selectable_manufacturers_inline(request, manufacturer_id)),
+        ("#selectable-manufacturers", selectable_manufacturers_inline(request, manufacturer_id)),
     )
 
     result = simplejson.dumps({
-        "html" : html,
-        "message" : msg
-    }, cls = LazyEncoder)
+        "html": html,
+        "message": msg
+    }, cls=LazyEncoder)
 
     return HttpResponse(result)
 
